define(['jquery', 'analytics'], function ($, analytics) {
	"use strict";

		// Service server (defaults to //directory.unl.edu)
	var directoryServer = null;

	var unlContext = '015236299699564929946:nk1siew10ie';

	var transitionDelay = 400;

	var inputSel = '#search_q';
	var formSel = '#searchform form';
	var resultSel = '.search-results';
	var googleSel = '.google-results';

	var evtStateChange = 'statechange';
	var wrapperMain = '#search_wrapper';
	var wrapperWeb = '#search_results';
	var wrapperDir = '#directory_results';

	var dirResults = 'ppl_results';
	var unlResults = 'unl_results';
	var localResults = 'local_results';

	window.pf_getUID = function() {
		return true;
	};

	// Caching Class
	var Cache = function() {
		this.storage = {};
	};
	Cache.prototype.get = function(key) {
		return this.storage[key] || undefined;
	};
	Cache.prototype.save = function(key, value) {
		this.storage[key] = value;
		return this;
	};

	// Directory Controller Class
	var Directory = function(server, containerId) {
		var cntSel = '#' + containerId;

		this._server = server || '//directory.unl.edu';
		this._cache = new Cache();
		this._searchCanceled = false;
		this._viewState = 0;
		this._renderTo = cntSel;

		$(function() {
			$(cntSel).on('click', '.fn a', function() {
				if (this.target !== '_blank') {
					this.target = '_blank';
				}
			});
		});
	};
	Directory.prototype._render = function(data) {
		if (this._searchCanceled) {
			return;
		}

		$(this._renderTo)
			.html(data)
			.addClass('active');

		this._renderState(0);
	};
	Directory.prototype._renderState = function(duration) {
		var $innerRes = $('.results', $(this._renderTo)),
			$showRes, failState,
			depFilter = '.departments';

		if (!$innerRes.length) {
			return;
		}

		$innerRes.slideUp(duration);
		if (this._viewState === 0) {
			$showRes = $innerRes.not(depFilter);
			failState = 1;
		} else {
			$showRes = $innerRes.filter(depFilter);
			failState = 0;
		}

		if (!$showRes.length && typeof duration !== "undefined") {
			this.changeViewState(failState);
			return;
		}

		$showRes.slideDown();
	};
	Directory.prototype.cancelSearch = function() {
		this._searchCanceled = true;
		if (this._xhr) {
			this._xhr.abort();
		}
	};
	Directory.prototype.execute = function(q) {
		var cacheData = this._cache.get(q),
			self = this;

		this._searchCanceled = false;
		if (this._xhr) {
			this._xhr.abort();
		}

		if (cacheData) {
			this._render(cacheData);
		} else {
			this._xhr = $.get(this._server + '/service.php?q=' + encodeURIComponent(q), function(data) {
				self._cache.save(q, data);
				self._render(data);
			});
		}
	};
	Directory.prototype.changeViewState = function(state) {
		if (this._viewState == state) {
			return;
		}

		var prevState = this._viewState;

		this._viewState = state;
		this._renderState();

		$(this._renderTo).trigger(evtStateChange, [state, prevState]);
	};
	Directory.prototype.clearAllResults = function() {
		$(this._renderTo).empty();
	};

	return {
		initialize: function(firstQ, localContext) {
			// query related
			var query = '';
			var actCls = 'active';

				// CustomSearchControl instances and config
			var unlSearch;
			var localSearch;
			var activeSearch;
			var directorySearch;
			var drawOp = new google.search.DrawOptions();
			var searchToggleLock = false;

			var trackQuery = function(q) {
					var loc = window.location,
						qs = loc.search.replace(/(?:(\?)|&)q=[^&]*(?:&|$)/, '$1'),
						page = [
						    loc.pathname,
						    qs || '?',
						    (qs && qs != '?') ? '&' : '',
						    'q=',
						    encodeURIComponent(q)
				        ].join('');

					analytics.callTrackPageview(page);

					if (window.history.pushState) {
						window.history.pushState({query: q}, '', page);
					}
			};

			var queryComplete = function(control) {
					var $root = $(control.root);

					// a11y patching
					$('img.gs-image', $root).each(function() {
						if (!this.alt) {
							this.alt = $(this).closest('.gsc-table-result').find('.gs-title').first().text();
						}
					});
					$('img.gcsc-branding-img-noclear', $root).attr('alt', 'Google™');

					if (!searchToggleLock && control == localSearch && $('.gs-no-results-result', $root).length) {
						$root.closest('.results-group').find('.result-tab li:last-child').click();
						return;
					}

					$root.closest(resultSel).addClass(actCls);
					$root.closest(googleSel).slideDown();

					searchToggleLock = false;
			};

			var fullQuery = function(q, track) {
					if (track !== false) {
						trackQuery(q);
					}
					try {
						activeSearch.execute(q, undefined, {});
					} catch (e) {
						queryComplete(activeSearch);
					}
					directorySearch.execute(q);
					$(wrapperMain).fadeIn();
			};

			var fullStop = function() {
					activeSearch.cancelSearch();
					directorySearch.cancelSearch();
					$(resultSel).removeClass(actCls);
					$(wrapperMain).fadeOut();
					setTimeout(function() {
						activeSearch.clearAllResults();
						directorySearch.clearAllResults();
					}, transitionDelay);
			};

			var queryStart = function(control, searcher, q) {
					$(control.root).closest(googleSel).slideUp(0);
					if (q !== query) {
						trackQuery(q);
						directorySearch.execute(q);
					}
			};

			drawOp.enableSearchResultsOnly();

			unlSearch = activeSearch = new google.search.CustomSearchControl(unlContext);
			unlSearch.setResultSetSize(google.search.Search.FILTERED_CSE_RESULTSET);
			unlSearch.setSearchCompleteCallback(window, queryComplete);
			unlSearch.setSearchStartingCallback(window, queryStart);

			if (localContext) {
				localSearch = activeSearch = new google.search.CustomSearchControl(localContext);
				localSearch.setResultSetSize('small');
				localSearch.setSearchCompleteCallback(window, queryComplete);
				localSearch.setSearchStartingCallback(window, queryStart);
			}

			directorySearch = new Directory(directoryServer, dirResults);

			// Setup DOM on ready
			$(function() {
				var $q = $(inputSel),

					resSel = '.results-group',
					tabsSel = '.result-tab',
					selCls = 'selected',
					stateClsPfx = 'state-',

					$resTabs = $('.result-tab'),
					tabStateChange = function(e, state, prevState) {
						var $tab = $(this).find(tabsSel);

						e.stopPropagation();

						$tab.removeClass(stateClsPfx + prevState);
						$tab.addClass(stateClsPfx + state);

						$tab.children().removeClass(selCls).eq(state).addClass(selCls);
					},

					googleOrigin = /^https?:\/\/.*\.google\.com$/,

					isValidOrigin = function(origin) {
						if (googleOrigin.test(origin)) {
							return false;
						}

						// don't allow self origin or browser extension origins
						if (origin == location.origin || /^chrome:/.test(origin)) {
							return false;
						}

						return true;
					},

					passiveQuery = function(q, track) {
						if (query === q) {
							return;
						}

						query = q;
						$q.val(q);

						if (q) {
							fullQuery(q, track);
						} else {
							fullStop();
						}
					};

				// draw the Google search controls
				unlSearch.draw(unlResults, drawOp);

				if (localContext) {
					localSearch.draw(localResults, drawOp);
				}

				// a11y patch Google search box
				$('form.gsc-search-box').remove();

				// setup the tab-like result filters
				$('li:first-child', $resTabs).addClass(selCls);
				$($resTabs).on('click', 'li', function(e) {
					e.preventDefault();

					if ($(this).hasClass(selCls)) {
						return;
					}

					var i = $(this).index(),
						$par = $(this).parents(resSel);

					if ($par.is(wrapperDir)) {
						directorySearch.changeViewState(i);
					} else if ($par.is(wrapperWeb)) {
						searchToggleLock = true;
						$(activeSearch.root).closest(googleSel).slideUp().trigger(evtStateChange, [i, 0 + !i]);
						if (i === 0) {
							activeSearch = localSearch;
						} else {
							activeSearch = unlSearch;
						}

						activeSearch.execute(query);
					}
				});

				$(resSel).on(evtStateChange, tabStateChange);

				// listen for the submit event
				$(formSel).submit(function(e) {
					e.preventDefault();

					var q = $.trim($q.val());
					passiveQuery(q);
				});

				// issue an inital query
				if (firstQ) {
					passiveQuery(firstQ, false);
				}

				// listen for message from parent frames
				$(window).on('message', function(e) {
					var oEvent = e.originalEvent, q;

					if (!isValidOrigin(oEvent.origin)) {
						return;
					}

					q = $.trim(oEvent.data);
					passiveQuery(q);
				});

				$(window).on('popstate', function(e) {
					var oEvent = e.originalEvent,
						q = firstQ || '';

					if (oEvent.state) {
						q = oEvent.state.query || '';
					}
					passiveQuery(q, false);
				});

				if (window.parent) {
					$(document).on('keydown', function(e) {
						if (e.keyCode === 27) {
							window.parent.postMessage('wdn.search.close', "*");
						}
					});
				}
			});
		}
	};
});
